@@ -6,9 +6,8 @@
 # For further information on the license, see the LICENSE.txt file.           #
 ###############################################################################
 """AiiDA-CP2K utils"""
-from __future__ import absolute_import
 
-from .input_generator import Cp2kInput
+from .input_generator import Cp2kInput, add_restart_sections
 from .parser import parse_cp2k_output
 from .parser import parse_cp2k_output_advanced
 from .parser import parse_cp2k_trajectory
